package org.jbpm.examples.identity;

import java.util.List;

import org.jbpm.api.Configuration;
import org.jbpm.api.ProcessInstance;
import org.jbpm.api.RepositoryService;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;


public class CustomIdentitySessionTest  extends JbpmTestCase {

  String jpdlXmlString = 
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
  "<process name=\"TaskCandidates\" xmlns=\"http://jbpm.org/4.3/jpdl\">" +
  "  <start g=\"20,20,48,48\">" +
  "    <transition to=\"review\" />" +
  "  </start>" +
  "  <task name=\"review\" " +
  "       candidate-groups=\"sales-dept\"" +
  "        g=\"96,16,127,52\">" +
  "     <transition to=\"wait\" />" +
  "  </task>" +
  "  <state name=\"wait\" g=\"255,16,88,52\"/>" +
  "</process>";
  
  @Override
  protected synchronized void initialize() {
    if (processEngine==null) {
      String xmlString = 
      "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
      "<jbpm-configuration>" +
      "  <import resource=\"jbpm.default.cfg.xml\" />" +
      "  <import resource=\"jbpm.businesscalendar.cfg.xml\" />" +
      "  <import resource=\"jbpm.tx.hibernate.cfg.xml\" />" +
      "  <import resource=\"jbpm.jpdl.cfg.xml\" />" +
      "  <import resource=\"jbpm.bpmn.cfg.xml\" />" +
      "  <import resource=\"jbpm.mail.templates.examples.xml\" />" +
      "  <transaction-context>" +
      "    <object class=\"org.jbpm.examples.identity.CustomIdentitySession\" />" +
      "  </transaction-context>" +
      "</jbpm-configuration>";
      processEngine = new Configuration().setXmlString(xmlString).buildProcessEngine();
      
      repositoryService = processEngine.get(RepositoryService.class);
      executionService = processEngine.getExecutionService();
      historyService = processEngine.getHistoryService();
      managementService = processEngine.getManagementService();
      taskService = processEngine.getTaskService();
      identityService = processEngine.getIdentityService();
    }
  }
  
  String deploymentId;
  String dept;

  protected void setUp() throws Exception {
    super.setUp();

    // create identities
    dept = identityService.createGroup("sales-dept");

    identityService.createUser("johndoe", "John", "Doe");
    identityService.createMembership("johndoe", dept, "developer");

    identityService.createUser("joesmoe", "Joe", "Smoe");
    identityService.createMembership("joesmoe", dept, "developer");

    // deploy process
    deploymentId = repositoryService.createDeployment()
        .addResourceFromString("xmlstring.jpdl.xml", jpdlXmlString)
        .deploy();
  }

  protected void tearDown() throws Exception {
    // delete process deployment
    repositoryService.deleteDeploymentCascade(deploymentId);

    // delete identities
    identityService.deleteGroup(dept);
    identityService.deleteUser("johndoe");
    identityService.deleteUser("joesmoe");

    super.tearDown();
  }

  public void testGroupCandidateAssignment() {
    ProcessInstance processInstance = executionService.startProcessInstanceByKey("TaskCandidates");
    String pid = processInstance.getId();

    // both johndoe and joesmoe will see the task in their *group* task list 
    List<Task> taskList = taskService.findGroupTasks("johndoe");
    assertEquals("Expected a single task in johndoe's task list", 1, taskList.size());
    Task task = taskList.get(0);
    assertEquals("review", task.getName());

    taskList = taskService.findGroupTasks("joesmoe");
    assertEquals("Expected a single task in joesmoe's group task list", 1, taskList.size());
    task = taskList.get(0);
    assertEquals("review", task.getName());

    // johndoe and joesmoe will NOT see the task in their *assigned* task list 

    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals(0, taskList.size());

    taskList = taskService.findPersonalTasks("joesmoe");
    assertEquals(0, taskList.size());

    // lets assume that johndoe takes the task
    taskService.takeTask(task.getId(), "johndoe");

    // johndoe's and joesmoe's group task list is now empty  
    taskList = taskService.findGroupTasks("johndoe");
    assertEquals(0, taskList.size());

    taskList = taskService.findGroupTasks("joesmoe");
    assertEquals(0, taskList.size());

    // johndoe's assigned task list has the task in it
    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals("Expected a single task being created", 1, taskList.size());
    task = taskList.get(0);
    assertEquals("review", task.getName());
    assertEquals("johndoe", task.getAssignee());

    // joesmoe will not see the task in the assigned task list
    taskList = taskService.findPersonalTasks("joesmoe");
    assertEquals(0, taskList.size());

    // now johndoe completes the task
    taskService.completeTask(task.getId());

    // verify that the task list is now empty
    taskList = taskService.findPersonalTasks("johndoe");
    assertEquals(0, taskList.size());

    // verify that process moved to the next state
    processInstance = executionService.findProcessInstanceById(pid);
    assertTrue(processInstance.isActive("wait"));
  }

}
